local addon = LibStub("AceAddon-3.0"):GetAddon("RCLootCouncil")
local RCEPGP = addon:NewModule("RCEPGP", "AceComm-3.0", "AceConsole-3.0", "AceHook-3.0", "AceEvent-3.0")
local EPGP = LibStub("AceAddon-3.0"):GetAddon("EPGP")
local GS = LibStub("LibGuildStorage-1.2")

local votingFrame = addon:GetActiveModule("votingframe")
local originalCols = {unpack(votingFrame.scrollCols)}

function RCEPGP:OnInitialize()
   self.version = GetAddOnMetadata("RCLootCouncil_EPGP", "Version")
   self:Enable()
end

function RCEPGP.UpdateVotingFrame()
  votingFrame:Update()
end

function RCEPGP:OnEnable()
   EPGP.RegisterCallback(self, "StandingsChanged",
                        RCEPGP.UpdateVotingFrame)
   self:SetupColumns()
end

function RCEPGP:OnDisable()
   -- Reset cols
   votingFrame.scrollCols = originalCols
end


function RCEPGP:SetupColumns()
   local ep =
   { name = "EP", DoCellUpdate = self.SetCellEP, colName = "ep", sortnext = self.GetScrollColIndexFromName("response"), width = 60, align = "CENTER", }
   local gp =
   { name = "GP", DoCellUpdate = self.SetCellGP, colName = "gp", sortnext = self.GetScrollColIndexFromName("response"), width = 50, align = "CENTER", }
   local pr =
   { name = "PR", DoCellUpdate = self.SetCellPR, colName = "pr", sortnext = self.GetScrollColIndexFromName("response"), width = 50, align = "CENTER", comparesort = self.PRSort,}
   table.insert(votingFrame.scrollCols, ep)
   table.insert(votingFrame.scrollCols, gp)
   table.insert(votingFrame.scrollCols, pr)
end

function RCEPGP:GetScrollColIndexFromName(name)
   for i,v in ipairs(votingFrame.scrollCols) do
      if v.colName == name then
         return i
      end
   end
end

---------------------------------------------
-- Lib-st UI functions
---------------------------------------------
function RCEPGP.SetCellEP(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
    local name = data[realrow].name
    local ep, gp, main = EPGP:GetEPGP(name)
    frame.text:SetText(ep or 0)
    data[realrow].cols[column].value = ep or 0
end

function RCEPGP.SetCellGP(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
    local name = data[realrow].name
    local ep, gp, main = EPGP:GetEPGP(name)
    frame.text:SetText(gp or 0)
    data[realrow].cols[column].value = gp or 0
end

function RCEPGP.SetCellPR(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
    local name = data[realrow].name
    local ep, gp, main = EPGP:GetEPGP(name)
    local pr = 0
    if ep and gp then
        pr = ep / gp
    end
    frame.text:SetText(string.format("%.4f", pr))
    data[realrow].cols[column].value = pr or 0
end

function RCEPGP.PRSort(table, rowa, rowb, sortbycol)
  local column = table.cols[sortbycol]
  local a, b = table:GetRow(rowa), table:GetRow(rowb);
  -- Extract the rank index from the name, fallback to 100 if not found

  local a_ep, a_gp = EPGP:GetEPGP(a.name)
  local b_ep, b_gp = EPGP:GetEPGP(b.name)

  a_ep = a_ep or 0
  a_gp = a_gp or 1
  b_ep = b_ep or 0
  b_gp = b_gp or 1

  local a_pr = a_ep/a_gp
  local b_pr = b_ep/b_gp

  local a_qualifies = a_ep >= EPGP.db.profile.min_ep
  local b_qualifies = b_ep >= EPGP.db.profile.min_ep

  if a_qualifies == b_qualifies and a_pr == b_pr then
    if column.sortnext then
      local nextcol = table.cols[column.sortnext];
      if nextcol and not(nextcol.sort) then
        if nextcol.comparesort then
          return nextcol.comparesort(table, rowa, rowb, column.sortnext);
        else
          return table:CompareSort(rowa, rowb, column.sortnext);
        end
      end
    end
    return false
  else
    local direction = column.sort or column.defaultsort or "asc";
    if direction:lower() == "asc" then
      if a_qualifies == b_qualifies then
        return a_pr < b_pr
      else
        return b_qualifies
      end
    else
      if a_qualifies == b_qualifies then
        return a_pr > b_pr
      else
        return a_qualifies
      end
    end
  end
end